/*
 * Copyright (c) 1999-2019 Hartmut Seichter
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
#ifndef PW_CORE_AXISANGLE_HPP
#define PW_CORE_AXISANGLE_HPP

#include <pw/core/vector.hpp>

namespace pw {

template <typename T>
class axisangle_ {
protected:
    vector3_<T> _axis;
    T _angle;
public:

    axisangle_()
        : _axis(vector3_<T>::up()),
          _angle(0)
    {}

    axisangle_(const vector3_<T> &axis,const T &angle)
        : _axis(axis)
        , _angle(angle)
    {
    }

    vector3_<T> axis() const { return _axis; }
    void set_axis(const vector3_<T> &axis) { _axis = axis; }

    T angle() const { return _angle; }
    void set_angle(const T &angle) { _angle = angle; }

    matrix4x4_<T> to_matrix() const
    {
        using std::cos;
        using std::sin;

        matrix4x4_<T> R; R.set_identity(); // = matrix44<T>::Identity();

        if (_axis.norm() < std::numeric_limits<T>::epsilon()) return R;

        const T _fCos = cos(_angle);

		vector3_<T> _vCos(_axis * (1 - _fCos));
		vector3_<T> _vSin(_axis * sin(_angle));

//        R.at(0) = (_axis(0,0) * _vCos(0,0)) + _fCos;
        //        R.at(4) = (T) ((vec(0,0) * _vCos(1,0)) - _vSin(2,0));
        //        R.at(8) = (T) ((vec(0,0) * _vCos(2,0)) + _vSin(1,0));

        //        R.at(1) = (T) ((vec(1,0) * _vCos(0,0)) + _vSin(2,0));
        //        R.at(5) = (T) ((vec(1,0) * _vCos(1,0)) + _fCos);
        //        R.at(9) = (T) ((vec(1,0) * _vCos(2,0)) - _vSin(0,0));

        //        R.at(2) = (T)  ((vec(2,0) * _vCos(0,0)) - _vSin(1,0));
        //        R.at(6) = (T)  ((vec(2,0) * _vCos(1,0)) + _vSin(0,0));
        //        R.at(10)= (T)  ((vec(2,0) * _vCos(2,0)) + _fCos);

        return R;
    }



};


typedef axisangle_<real_t> axisangle;
typedef axisangle_<double> axisangled;
typedef axisangle_<float> axisanglef;

}





#endif
