#include "pw/visual/mesh_renderer.hpp"

#include "pw/core/mesh.hpp"
#include "pw/core/size.hpp"

#include "glad/glad.h"

#include <algorithm>

namespace pw {


struct mesh_renderer::impl {

	GLuint _vao = 0;
	std::vector<GLuint> _vbos;

	impl(mesh_renderer& )
	{
	}

	void create(std::shared_ptr<mesh> mesh)
	{
		glGenVertexArrays(1,&_vao);
		glBindVertexArray(_vao);

		size_t arrays_needed = 0;

		// should bail out here
		if (!mesh->vertices().empty()) arrays_needed++;
		if (!mesh->indices().empty()) arrays_needed++;

		// TODO: implement the other arrays

		_vbos.resize(arrays_needed);

		glGenBuffers(_vbos.size(), _vbos.data());

		// vertices
		glBindBuffer(GL_ARRAY_BUFFER, _vbos[0]);
		glBufferData(GL_ARRAY_BUFFER, sizeof(mesh->vertices().front()) * mesh->vertices().size(), mesh->vertices().data(),
					 GL_STATIC_DRAW);
		glEnableVertexAttribArray(0);
		glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, 0, nullptr);

		// indices
		glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, _vbos[1]);
		glBufferData(GL_ELEMENT_ARRAY_BUFFER, sizeof(mesh->indices().front()) * mesh->indices().size(), mesh->indices().data(),
				GL_STATIC_DRAW);


		// stop binding
		glBindVertexArray(0);

	}

	void destroy()
	{
		glDeleteVertexArrays(1,&_vao);
		for (auto vbo : _vbos)
			glDeleteBuffers(1,&vbo);

		_vbos.clear();
	}
};




}
